package mvd.jester.simulator.internals;

import java.util.HashSet;
import java.util.Optional;
import java.util.Set;
import mvd.jester.model.Segment;

/**
 * Segment
 */
public class SegmentContext {
    private final Segment segment;
    private final Set<JobContext> jobs;
    private final Set<TaskletContext> tasklets;

    public SegmentContext(TaskContext taskContext, Segment segment) {
        this.segment = segment;
        jobs = new HashSet<>();
        tasklets = new HashSet<>();

        for (int j = 0; j < segment.getNumberOfJobs(); ++j) {
            jobs.add(new JobContext(taskContext, this));
        }

        for (int t = 0; t < segment.getNumberOfTasklets(); ++t) {
            tasklets.add(new TaskletContext(this));
        }
    }


    /**
     * @return the segment
     */
    public Segment getSegment() {
        return segment;
    }

    Optional<JobContext> getNextJob() {
        return jobs.stream()
                .filter(j -> !j.getCurrentProcessor().isPresent() && j.checkExecutionTime())
                .findFirst();
    }


    @Override
    public String toString() {
        return "(nJobs=" + segment.getNumberOfJobs() + ", jobWcet=" + segment.getJobWcet() + ")";
    }
}
